#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include "graph.h"
#include "chil.h"
#include "math.h"
/* #include "helvetica.xbm" */

static char *months[]={"Jan","Feb","Mar","Apr","May","June","July","Aug","Sept","Oct","Nov","Dec"};

#define MAX_AXES 3
#define NO_SIG_VAL -999.0

FileList filelist;
Axes axes[MAX_AXES];
Colourmap map[MAX_AXES];
ChilboltonFile chil_file;
RasterInfo raster_info;
Ray ray;
int hour,minute,naxes=0;
unsigned short avegates=1,averays=24;
unsigned short int reqday=0,reqmonth=0,reqyear=0,len,firstraster,lastraster,raster,
   first_pixel=0;

void flush_and_exit() {

int i,axis,namelen;
char buf[127];
char fieldname[10],filename[16];
FILE *graphfile;

CHILClose(&chil_file);
CHILFreeRay(&ray);

for (axis=0;axis<naxes;axis++) {
   for (i=0;i<13;i+=2) {
      sprintf(buf,"%d",i);
      GraphYTick(&axes[axis],i/(((float) raster_info.gate_spacing * avegates)/1000.0),-5,buf);
   }
   if (reqday>0) {
      for (i=0;i<=24;i+=2) {
         sprintf(buf,"%02d:00",i);
         GraphXTick(&axes[axis],i*30,-5,buf);
      }
   }
/*
   sprintf(buf,"Last ray %02d:%02d UTC",hour,minute);
   GraphHText(&axes[axis],700,10,buf);
*/

   CHILFieldName(&ray,axis,fieldname);
   GraphHText(&axes[axis],700,245,fieldname);
   strcpy(filename,fieldname);
   namelen=strlen(filename);
   strcpy(filename+namelen,".xpm\0");
   graphfile=fopen(filename,"w");
   GraphWriteXpm(&axes[axis],&map[axis],graphfile,fieldname);
   GraphFreeAxes(&axes[axis]);
   fclose(graphfile);
}
exit(0);

}

int main(int argc,char **argv) {

int cloudgate;
float stdn;
int colour;
float minval[MAX_AXES],maxval[MAX_AXES],interval[MAX_AXES],
   conv[MAX_AXES],cal[MAX_AXES];
float v_1=-5,v_2=5;
float sv_1=0,sv_2=2.00001;
float Zcalibration=+25,noisefactor=3.0;
float utc,utc_old=0;
int axis,i=0,j=0,status,firstray=1,ifile,iarg;
int c1=2;
int c2=63;
int hour_old=0,day_old;
char buf[127],units[10],oscillator=0;

if (argc < 2) {
   fprintf(stderr,"Usage:  %s [-date YYYYMMDD] [-avegates n] [-averays m] filepattern1 [filepattern2 ...] \n",argv[0]);
   exit(1);
}

CHILSecondarySumDir("/export/anvil/anvil-01/swrhgnrj/sum");

iarg=1;
while ( iarg < argc ) {
   if (*argv[iarg] == '-') {
      if (strcmp(argv[iarg],"-avegates") == 0 && iarg<argc-1) {
         avegates=(unsigned short) atoi(argv[++iarg]);
      }
      else if (strcmp(argv[iarg],"-averays") == 0&& iarg<argc-1) {
         averays=(unsigned short) atoi(argv[++iarg]);
      }
      else if (strcmp(argv[iarg],"-cal") == 0 && iarg<argc-1) {
         Zcalibration=atof(argv[++iarg]);
      }
      else if (strcmp(argv[iarg],"-sumdir") == 0 && iarg<argc-1) {
         CHILSecondarySumDir(argv[++iarg]);
      }
      else if (strcmp(argv[iarg],"-noisefactor") == 0 && iarg<argc-1) {
         noisefactor=atof(argv[++iarg]);
      }
      else if (strcmp(argv[iarg],"-oscillator") == 0) {
         oscillator=1;
      }
      else if (strcmp(argv[iarg],"-date") == 0 && iarg<argc-1) {
         i=0;
         if ((len=strlen(argv[++iarg]))==6) {
            i=sscanf(argv[iarg],"%02hu%02hu%02hu",&reqyear,&reqmonth,&reqday);
         }
         else if (len==8) {
            i=sscanf(argv[iarg],"%04hu%02hu%02hu",&reqyear,&reqmonth,&reqday);
            reqyear -= 1900;
         }
         if (i != 3) {
            fprintf(stderr,"Date not understood: should be in the form yymmdd or yyyymmdd\n");
            exit(-1);
         } 
      }
      else
         fprintf(stderr,"Option `%s' not understood\n",argv[iarg]);
      iarg++;
   }
   else 
      break;
}

if (reqday != 0) {
   CHILGetFileList(reqday,reqmonth,reqyear,argc-iarg,argv+iarg,&filelist);
   iarg=0;
   argc=filelist.n;
   argv=filelist.name;
   fprintf(stderr,"Found %hu files for %hu/%hu/%hu\n",(unsigned short int) filelist.n,reqday,reqmonth,reqyear);
}

signal(SIGINT,flush_and_exit);
signal(SIGTERM,flush_and_exit);

for (ifile=iarg;ifile<argc;ifile++) {

if (CHILOpen(argv[ifile],&chil_file) == FILE_CLOSED) {
   fprintf(stderr,"Error reading %s: aborting\n",argv[ifile]);
   exit(-1);
}

if (reqday == 0) {
   firstraster=chil_file.raster[0];
   lastraster=chil_file.raster[chil_file.nrasters-1];
}
else {
   firstraster=filelist.first[ifile];
   lastraster=filelist.last[ifile];
}

//CHILPrintFileInformation(&chil_file);

for (raster=firstraster;raster<lastraster;raster++) {
if (CHILSeekRaster(&chil_file,raster) !=0 ) {
   fprintf(stderr,"Warning: raster not found\n");
   break;
}

status=CHILReadInformationRecord(&chil_file,&raster_info);
//CHILPrintInformationRecord(&raster_info);

if (firstray) {
   CHILInitRay(&raster_info,avegates,averays,&ray);
   if (naxes==0) {
      if ((naxes=ray.ntotal) > MAX_AXES) naxes=MAX_AXES;
   }
   for (axis=0;axis<naxes;axis++) {
      GraphCreateAxes(&axes[axis],825,265,720,200,35,35);
      GraphHText(&axes[axis],350,10,"Time (UTC)");
      GraphVText(&axes[axis],10,110,"Height (km)");


      switch (ray.field_type[axis]) {
         case ZH:
#ifdef DEBUG 
   fprintf(stderr,"Initialising Z: ");
#endif
            cal[axis]=Zcalibration;
            minval[axis]=-45;maxval[axis]=15;interval[axis]=10;
            GraphCreateJet(&map[axis]);
            break;
         case V1:case V2:
#ifdef DEBUG 
   fprintf(stderr,"Initialising v: ");
#endif
            cal[axis]=0;
            minval[axis]=v_1;maxval[axis]=v_2;interval[axis]=1;
            GraphCreateLitmus(&map[axis]);
//            GraphCreateJet(&map[axis]);
            break;
         case V_STD:
#ifdef DEBUG 
   fprintf(stderr,"Initialising sigma-v: ");
#endif
            cal[axis]=0;
            minval[axis]=sv_1;maxval[axis]=sv_2;interval[axis]=0.4;
            GraphCreateJet(&map[axis]);
            break;
         default:
            fprintf(stderr,"Fatal error: don't know how to draw parameter #%d\n",axis);
            exit(1);
      }

#ifdef DEBUG 
   fprintf(stderr,"properties...");
#endif
      conv[axis]=((float) (c2-c1))/(maxval[axis]-minval[axis]);
      GraphDrawVColourbar(&axes[axis],&map[axis],800,35,15,170,
               minval[axis],maxval[axis],interval[axis],"%4.1f",LINEAR);
#ifdef DEBUG 
   fprintf(stderr,"units...");
#endif
      CHILFieldUnits(&ray,axis,units);
#ifdef DEBUG 
   fprintf(stderr,"misc...");
#endif
      GraphHText(&axes[axis],790,225,units);
      sprintf(buf,"Chilbolton %2.0f GHz cloud radar",raster_info.frequency);
      GraphHText(&axes[axis],25,10,buf);
#ifdef DEBUG 
   fprintf(stderr,"done\n");
#endif
   }
}
else
   CHILReInitRay(&raster_info,avegates,averays,&ray);

#ifdef DEBUG 
   fprintf(stderr,"Reading ray\n");
#endif
while ((status=CHILReadRay(&chil_file,&raster_info,&ray)) == 0 && j < axes[0].xaxissize && ~feof(chil_file.file)) {
   if (reqday ==0 || (ray.day == reqday && ray.month == reqmonth && ray.year == reqyear)) {

#ifdef DEBUG 
   fprintf(stderr,"Subtracting noise\n");
#endif
   if (oscillator) {
      if (CHILSubtractOscillatorNoise(&ray,25,noisefactor,NO_SIG_VAL) != 0 ) {
         fprintf(stderr,"Fatal error: cannot find Z\n");
         exit(1);
      }
   }
   else {
      if (CHILSubtractNoise(&ray,25,noisefactor,NO_SIG_VAL) != 0 ) {
         fprintf(stderr,"Fatal error: cannot find Z\n");
         exit(1);
      }
   }


   CHILCleanRay(&ray,NO_SIG_VAL);
   CHILRangeCorrection(&raster_info,&ray,NO_SIG_VAL);
   utc=ray.utc;
   hour=(int) utc;
   minute=(utc-((float) hour))*60;
   if (firstray) {
      firstray=0;
      utc_old=utc;
      hour_old=hour;
      day_old=raster_info.day-1;
      /* find the first pixel that has a positive range */
      i=-1;
      while (*(ray.range + (++i))+ray.range_offset*0.001 < 0.0);
      first_pixel=i;
   }

   if ((utc-utc_old>0.25 || (utc<utc_old && utc>utc_old-23.75) ) && reqday == 0) {
      j+=10;
      for (axis=0;axis<naxes;axis++)
         GraphDrawVLine(&axes[axis],axes[axis].xoffs+j-1,axes[axis].yoffs,axes[axis].yaxissize,BLACK);
   }
#ifdef DEBUG 
#endif

   if (reqday > 0)
      j=ray.utc*30;

   for (i=first_pixel;i<ray.ngates;i++) {
      for (axis=0;axis<naxes;axis++) {
         colour=(*(ray.data[axis]+i)-minval[axis]+cal[axis]) * conv[axis] + c1;
         if  (colour<c1) colour=c1;
         else if  (colour>c2) colour=c2;
         if (*(ray.data[axis]+i) != NO_SIG_VAL) 
            GraphDrawPixel(&axes[axis],j,i-first_pixel,colour);
      }
   }
#ifdef DEBUG 
#endif
   if (hour != hour_old && (hour % 2) == 0 && reqday == 0) {
      if (minute < 5)
         sprintf(buf,"%02d:00",hour);
      else
         sprintf(buf,"%02d:%02d",hour,minute);
      for (axis=0;axis<naxes;axis++) GraphXTick(&axes[axis],j,-5,buf);
      if (day_old != raster_info.day) {
         for (axis=0;axis<naxes;axis++) 
            GraphDrawVLine(&axes[axis],j+axes[axis].xoffs,axes[axis].yoffs+axes[axis].yaxissize,15,BLACK);
      }
   }
#ifdef DEBUG 
#endif
   if (day_old != raster_info.day) {
      if (reqday>0)
         j=0;
      sprintf(buf,"%d %s %d",raster_info.day,months[raster_info.month-1],raster_info.year+1900);
      for (axis=0;axis<naxes;axis++) 
         GraphHText(&axes[axis],j+axes[axis].xoffs+3,245,buf);
   }
   hour_old=hour;
   utc_old=utc;
   day_old=raster_info.day;
   j++;
fprintf(stderr,"#");
} // end of ray
else {
fprintf(stderr,".");
//   fprintf(stderr,"(:%f %hu %hu %hu %hu %hu %hu) ",ray.utc,ray.day,reqday,ray.month,reqmonth,ray.year,reqyear);
}
}
} // end of raster
CHILClose(&chil_file);
} // end of file
fprintf(stderr,"\n");

flush_and_exit();
}


