/* dirchil.c
 * Matlab tool for obtaining the dates available in a set of chilbolton files,
 * or displaying the rasters that were taken on a 
 * particular date in a set of chilbolton data files
 */
#include <stdio.h>
#include <glob.h>
#include <string.h>
#include <mex.h>
#include "mextools.h"
#include "chil.h"

void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[]) {
   FileList filelist;
   int datelen,i,pos=0,status;
   unsigned short int day=0,month=0,year=0;
   unsigned short int pday=0,pmonth=0,pyear=0;
   char **argv,*argt,isdate=0;
   glob_t globbuf;
   unsigned int GLOB_FLAG=0;
   FILE *sumfile;
   char tmp[255];

   CHILSecondarySumDir("/export/anvil/anvil-01/swrhgnrj/sum");

   if (nrhs==0) {
      mexPrintf("Usage: dirchil file_pattern1 [file_pattern2 ...]\n       dirchil -date yyyymmdd file1 [file2...]\n");
      return;
   }

   MexConvertStringArguments(nrhs,prhs,&argv,&argt);

   for (i=0;i<nrhs;i++) {
      if (strcmp(argv[i],"-date")==0) {
         isdate=1;
         break;
      }
      if ((status=glob(argv[i],GLOB_FLAG,NULL,&globbuf)) == GLOB_NOSPACE) {
         globfree(&globbuf);
         MexFreeArguments(argv);
         mexErrMsgTxt("Out of memory");
      }
      GLOB_FLAG=GLOB_APPEND;
   }

   if (isdate) {
      if (i==0) {
         globfree(&globbuf);
         MexFreeArguments(argv);
         mexErrMsgTxt("No files specified");
      }
      if (i==nrhs-1) {
         globfree(&globbuf);
         MexFreeArguments(argv);
         mexErrMsgTxt("No date specified");
      }
      datelen=strlen(argv[++i]);
      if (datelen==6) {
         sscanf(argv[i],"%02hu%02hu%02hu",&year,&month,&day);
      }
      else if (datelen==8) {
         sscanf(argv[i],"%04hu%02hu%02hu",&year,&month,&day);
         year=year-1900;
      }
      else {
         globfree(&globbuf);
         MexFreeArguments(argv);
         mexErrMsgTxt("Can't understand date");
      }
      CHILGetFileList(day,month,year,globbuf.gl_pathc,globbuf.gl_pathv,&filelist);
      for (i=0;i<filelist.n;i++) {
         mexPrintf("%s %hu-%hu ",filelist.name[i],filelist.first[i],filelist.last[i]);
      }
      mexPrintf("\n");
   }
   else {
      for (i=0;i<globbuf.gl_pathc;i++) {
         if ((sumfile=CHILOpenSummary(globbuf.gl_pathv[i])) != NULL) {
            fgets(tmp,255,sumfile); /* skip first line */
            while(fgets(tmp, 255, sumfile) != '\0') {
               sscanf(tmp, "%*5d %*3hu %*8c %2hu/%2hu/%2hu",&day,&month,&year);
               if (day != pday || month != pmonth || year != pyear) {
                  mexPrintf("%02hu%02hu%02hu \n",year,month,day);
                  pday=day;pmonth=month;pyear=year;
               }
            }
            fclose(sumfile);
         }
      }
   }
   globfree(&globbuf);
   MexFreeArguments(argv);
}
