/*
graph.c
*/

#include "graph.h"
#include "helvetica.xbm"
#include <string.h>

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
static unsigned short int helvetica_sizes[]={0,4,7,11,19,26,37,46,49,52,57,62,69,73,81,83,88,95,102,109,115,123,130,137,144,151,158,160,163,171,178,185,192,204,213,221,230,239,247,255,264,273,276,283,291,298,309,318,328,336,346,354,361,369,377,386,397,406,415,424,426,430,433,439,446,450,457,464,471,478,484,488,495,502,504,508,514,517,526,533,540,547,554,558,563,567,574,580,589,596,603,608,613,615,619,628};
int GraphCreateAxes(Axes *axes,uword xsize,uword ysize,uword xaxissize,uword yaxissize,uword xoffs,uword yoffs) {
unsigned char *raw_ptr,**mem_ptr;
uword i,j;

axes->xsize=xsize;
axes->ysize=ysize;
axes->xoffs=xoffs;
axes->yoffs=yoffs;
axes->xaxissize=xaxissize;
axes->yaxissize=yaxissize;

/* allocate memory for raw raster data */
raw_ptr=malloc(xsize*ysize);
/* allocate memory for pointers to raster data */
mem_ptr=calloc(ysize+yaxissize,sizeof(mem_ptr));
/* set pointers to raster data */
axes->raw=(void *) mem_ptr;
axes->pixel=(void *) (mem_ptr+ysize);
for (i=0;i<ysize;i++) {
   *(axes->raw+i) = raw_ptr+i*xsize;
}
for (i=0;i<yaxissize;i++) {
   *(axes->pixel+i) = axes->raw[i+yoffs]+xoffs;
}
/* set area to WHITE */
for (i=0;i<ysize;i++) {
   for (j=0;j<xsize;j++) {
      *(*(axes->raw+i)+j)=WHITE;
   }
}
/* draw axes lines */
GraphDrawHLine(axes,xoffs,yoffs,xaxissize,BLACK);
GraphDrawHLine(axes,xoffs,yoffs+yaxissize,xaxissize,BLACK);
GraphDrawVLine(axes,xoffs,yoffs,yaxissize,BLACK);
GraphDrawVLine(axes,xoffs+xaxissize,yoffs,yaxissize,BLACK);
}

int GraphFreeAxes(Axes *axes) {
free((void *) *(axes->raw));
free((void *) axes->raw);
}

int GraphDrawRawPixel(Axes *axes,uword x,uword y,char colour) {
if (x<axes->xsize && y<axes->ysize) {
   *(axes->raw[y]+x)=colour;
   return 0;
}
else 
   return 1;
}
int GraphDrawPixel(Axes *axes,uword x,uword y,char colour) {
if (x<axes->xaxissize-1 && y<axes->yaxissize-1 && x>0 && y>0) {
   *(axes->pixel[y]+x)=colour;
   return 0;
}
else 
   return 1;
}
int GraphDrawHLine(Axes *axes,uword xstart,uword ystart,uword length,char colour) {
uword i;
for (i=0;i<=length;i++) {
   GraphDrawRawPixel(axes,xstart+i,ystart,colour);
}
}
int GraphDrawVLine(Axes *axes,uword xstart,uword ystart,uword length,char colour) {
uword i;
for (i=0;i<=length;i++) {
   GraphDrawRawPixel(axes,xstart,ystart+i,colour);
}
}

int GraphHText(Axes *axes,uword x,uword y,char * text) {
uword ch=0,i,j;
uchar ichar,byte;
while (text[ch] != '\0' ) {
   if (text[ch]>' ' && text[ch] <= '~') {
      ichar=text[ch]-32;
      for (j=helvetica_sizes[ichar];j<helvetica_sizes[ichar+1];j++) {
         for (i=0;i<helvetica_height;i++) {
            byte=helvetica_bits[i*((helvetica_width+7)/8) +j/8];
            if (byte & (1 << (j % 8))) {
               GraphDrawRawPixel(axes,x,y-i+(helvetica_height/2),BLACK);
            }
         }
         x++;
      }
   }
   else {
      x+=helvetica_sizes[1];
   }
   ch++;
}
return 0;
}
int GraphVText(Axes *axes,uword x,uword y,char * text) {
uword ch=0,i,j;
uchar ichar,byte;
while (text[ch] != '\0' ) {
   if (text[ch]>' ' && text[ch] <= '~') {
      ichar=text[ch]-32;
      for (j=helvetica_sizes[ichar];j<helvetica_sizes[ichar+1];j++) {
         for (i=0;i<helvetica_height;i++) {
            byte=helvetica_bits[i*((helvetica_width+7)/8) +j/8];
            if (byte & (1 << (j % 8))) {
               GraphDrawRawPixel(axes,x+i-(helvetica_height/2),y,BLACK);
            }
         }
         y++;
      }
   }
   else {
      y+=helvetica_sizes[1];
   }
   ch++;
}
return 0;
}

uword GraphTextLength(char * text) {
uword ch=0,x=0;
uchar ichar;
while (text[ch] != '\0' ) {
   if (text[ch]>' ' && text[ch] <= '~') {
      ichar=text[ch]-32;
      x += (helvetica_sizes[ichar+1] - helvetica_sizes[ichar]);
   }
   else {
      x+=helvetica_sizes[1];
   }
   ch++;
}
return x;
}


int GraphXTick(Axes *axes,uword x,short int length,char *text) {
uword xpos;
xpos=axes->xoffs+x;
if (length < 0) {
/* Ticks go in */
GraphDrawVLine(axes,xpos,axes->yoffs,-length,BLACK);
GraphDrawVLine(axes,xpos,axes->yoffs+axes->yaxissize+length,-length,BLACK);
GraphHText(axes,xpos-GraphTextLength(text)/2,axes->yoffs-(helvetica_height/2)-1,text);
}
else {
/* Ticks go out */
GraphDrawVLine(axes,xpos,axes->yoffs-length,length,BLACK);
GraphDrawVLine(axes,xpos,axes->yoffs+axes->yaxissize,length,BLACK);
GraphHText(axes,xpos-GraphTextLength(text)/2,axes->yoffs-length-(helvetica_height/2),text);

}

}
int GraphYTick(Axes *axes,uword y,short int length,char *text) {
uword ypos;
ypos=axes->yoffs+y;
if (length < 0) {
/* Ticks go in */
GraphDrawHLine(axes,axes->xoffs,ypos,-length,BLACK);
GraphDrawHLine(axes,axes->xoffs+axes->xaxissize+length,ypos,-length,BLACK);
GraphHText(axes,axes->xoffs-GraphTextLength(text)-2,ypos,text);
}
else {
/* Ticks go out */
GraphDrawHLine(axes,axes->xoffs-length,ypos,length,BLACK);
GraphDrawHLine(axes,axes->xoffs+axes->xaxissize,ypos,length,BLACK);
GraphHText(axes,axes->xoffs-GraphTextLength(text)-2-length,ypos,text);
}

}
int GraphWriteXpm(Axes *axes,Colourmap * map,FILE *file,char *name) {
short int i,j;
fprintf(file,"/* XPM */\nstatic char *%s[] = {\n",name);
fprintf(file,"/* width height ncolours cpp */\n\"%hu %hu %hu 1\",\n/* colours */\n",axes->xsize,axes->ysize,map->ncolours);

for (i=0;i<map->ncolours;i++) {
   fprintf(file,"\"%c c #%02X%02X%02X\",\n",map->symbol[i],map->R[i],map->G[i],map->B[i]);
}
fprintf(file,"/* pixels */\n");
for (i=axes->ysize-1;i>=0;i--) {
   fprintf(file,"\"");
   for (j=0;j<axes->xsize;j++) {
      fprintf(file,"%c",(int) map->symbol[*(axes->raw[i]+j)]);
   }
   fprintf(file,"\",\n");
}
fprintf(file,"};\n");
return 0;
}

/* Rather than write the code to compress the data into gif format, we call the
program `convert' to do it for us. */
#ifdef WRITEGIF
int GraphWriteGif(Axes *axes,Colourmap * map,FILE *file) {
pid_t pid;
int pipes[2],status;
FILE *pipein,*pipeout;
pipe(pipes);
pipein=fdopen(pipes[0],"r");
pipeout=fdopen(pipes[1],"w");

if ((pid=fork()) == 0) {
   dup2(pipein->_file,stdin->_file);
   dup2(file->_file,stdout->_file);
   fprintf(stderr,"An error occurred in converting to gif: code%d\n",
         execlp("convert","convert","-","GIF:-",NULL));
//        execlp("cat","cat",NULL));
   exit(10);
}

status=GraphWriteXpm(axes,map,pipeout,"generic");
fclose(pipeout);
//fclose(pipein);
return status;
}
#endif
int GraphCreateColourmap(Colourmap *map,uword ncolours) {
uword i;

if (ncolours<2) {
   fprintf(stderr,"Cannot have a colourmap with fewer than 2 colours\n");
   return 1;
}
if (ncolours>MAX_COLOURS) {
   fprintf(stderr,"Cannot have a colourmap with more than %d colours\n",MAX_COLOURS);
   return 2;
}

map->ncolours=ncolours;
map->symbol[0]=35;
map->symbol[1]=36;
map->R[BLACK]=0;
map->G[BLACK]=0;
map->B[BLACK]=0;
map->R[WHITE]=255;
map->G[WHITE]=255;
map->B[WHITE]=255;

for (i=2;i<ncolours;i++) {
   map->symbol[i]=35+i;
   map->R[i]= (((ncolours-i+1) * 255) / ncolours);
   (map->G[i])=(map->R[i]);
   (map->B[i])=(map->R[i]);
}
return 0;
}

int GraphColourmapRange(Colourmap *map,uword i1,uword i2,uchar R1,uchar G1,uchar B1,uchar R2,uchar G2,uchar B2) {
uword i,isize;
isize=i2-i1;
for (i=i1;i<=i2;i++) {
   map->R[i]=(R2*(i-i1)+R1*(i2-i))/isize;
   map->G[i]=(G2*(i-i1)+G1*(i2-i))/isize;
   map->B[i]=(B2*(i-i1)+B1*(i2-i))/isize;
}
}
int GraphCreateJet(Colourmap *map) {
uword i;
GraphCreateColourmap(map,64);
/*
GraphColourmapRange(map,0+2,7+2,127,255,127,0,127,0);
GraphColourmapRange(map,7+2,23+2,0,127,0,0,0,255);
GraphColourmapRange(map,23+2,39+2,0,0,255,255,0,0);
GraphColourmapRange(map,39+2,55+2,255,0,0,255,255,0);
GraphColourmapRange(map,55+2,63+2,255,255,0,127,255,255);
*/
/* GraphColourmapRange(map,0+2,7+2,239,239,239,127,127,255); */
GraphColourmapRange(map,0+2,7+2,223,223,223,127,127,255);
GraphColourmapRange(map,7+2,15+2,127,127,255,0,0,255);
GraphColourmapRange(map,15+2,23+2,0,0,255,0,159,159);
GraphColourmapRange(map,23+2,31+2,0,159,159,0,255,0);
GraphColourmapRange(map,31+2,39+2,0,255,0,255,255,0);
GraphColourmapRange(map,39+2,55+2,255,255,0,255,0,0);
GraphColourmapRange(map,55+2,63+2,255,0,0,127,0,127);
/*
GraphColourmapRange(map,0+2,15+2,127,127,127,0,0,255);
GraphColourmapRange(map,15+2,31+2,0,0,255,0,255,0);
GraphColourmapRange(map,31+2,47+2,0,255,0,255,255,0);
GraphColourmapRange(map,47+2,63+2,255,255,0,255,0,0);
*/
}
int GraphCreateLitmus(Colourmap *map) {
uword i;
GraphCreateColourmap(map,64);
GraphColourmapRange(map,0+2,7+2,0,63,0,0,0,128);
GraphColourmapRange(map,7+2,15+2,0,0,128,0,0,255);
GraphColourmapRange(map,15+2,23+2,0,0,255,0,127,255);
GraphColourmapRange(map,23+2,29+2,0,127,255,0,255,255);
GraphColourmapRange(map,30+2,31+2,223,223,223,223,223,223);
GraphColourmapRange(map,32+2,47+2,255,255,0,255,0,0);
GraphColourmapRange(map,47+2,63+2,255,0,0,127,0,127);
}
int GraphDrawVColourbar(Axes *axes,Colourmap *map,uword x,uword y,uword width,uword height,float min,float max,float step,char * format,uchar scale) {

uword i,j;
uchar col;
float val;
char buf[16];

/* first do blocks of colour */
for (j=0;j<height;j++) {
   col=(j*(map->ncolours-3))/height+2;
   for (i=x;i<x+width;i++) {
      GraphDrawRawPixel(axes,i,j+y,col);
   }
}
/* now the border */
GraphDrawHLine(axes,x,y,width,BLACK);
GraphDrawHLine(axes,x,y+height,width,BLACK);
GraphDrawVLine(axes,x,y,height,BLACK);
GraphDrawVLine(axes,x+width,y,height,BLACK);

/* finally some Y Ticks */
for (val=min;val<=max;val+=step) {
   sprintf(buf,format,val);
   if (scale==LINEAR) {
/*
      GraphDrawHLine(axes,x-5,y,5,BLACK);
      GraphDrawHLine(axes,x+width,y,5,BLACK);
*/
/*     GraphDrawHLine(axes,x,y,width,BLACK); */
     GraphHText(axes,x-2-GraphTextLength(buf),y,buf);
   }
   else if (scale==LOGARITHMIC) {
      i=GraphTextLength("10");
      j=GraphTextLength(buf);
      GraphHText(axes,x-3-i-j,y,"10");
      GraphHText(axes,x-2-j,y+helvetica_height/3,buf);
   }
   else {
      fprintf(stderr,"Cannot understand scale for colourbar\n");
      return 1;
   }
   y+=((float) height*step)/(max-min);
}
return 0;
}

