function str=stripe(x,y,s,n,st,lt);

%stripe(x,y,s,n,st,lt): Fills in a region with stripes
% plots stripes where s=1
% x, y are vectors (or meshgrid-type matrices). s is a matrix
% n is the number of stripes (optional, unless lt is included) - default=30
% st is the 'stripetype' (optional):
%	/ \ | or -	default is \
% Thus cross hatching is achieved by calling stripe twice
% lt is the linestyle, as in plot (optional)
%
%Example:
%
%	[x y]=meshgrid(-2:.05:3);
%	z=peaks(x,y);
%	index=find(z>2);
%	s=zeros(size(z));
%	s(index)=ones(size(index));
%	contour(x,y,s,[-1000 1]);
%	stripe(x,y,s,35,'\','--r');

%Written by D. Chapman 10/6/1996

if nargin<3
error('Too few input arguments.');
elseif nargin==3
n=30;st='/';lt='';
elseif nargin==4
st='/';lt='';
elseif nargin==5
lt='';
end

hold_state = ishold;
hold on

ax=axis;
dis=(ax(2)-ax(1))/(ax(4)-ax(3));   %how much is the axis distorted?
si=size(x);
if min(si)==1
[xg yg]=meshgrid(x,y);
else
xg=x;yg=y;
end

%generate a height field for contouring....
if st=='/'
h=yg*dis-xg;
elseif st=='\'
h=xg+yg*dis;
elseif st=='-'
h=yg*dis;
elseif st=='|'
h=xg;
end

index=find(s~=1);
h(index)=NaN*ones(size(index));

%plot it all
if lt==''
contour(x,y,h,n);
else
contour(x,y,h,n,lt);
end

if hold_state==0, 
hold off
end











